
;; Part 5 - Derive a class that keeps track of how many searches have been 
;;    performed.
;; Nothing changes, but there's new class definition

(define lookup-number
  (lambda (name home?)
    (sleep 2) ; Simulate a delay
    (let loop ([book phone-book])
      (cond
	[(null? book) #f]
	[(string-ci=? name (caar book))
	 ((if home? cadar caddar) book)]
	[else (loop (cdr book))]))))

(define phone-book
  '(("Matthias" "555-1234" "527-8101 ext. 0000")
    ("Corky" "555-1357" "527-8101 ext. 9999")
    ("Robby" "555-0987" "527-8750 ext. 7777")
    ("Cormac" "555-8855" "527-8101 ext. 3333")
    ("Shriram" "555-1192" "527-8750 ext. 1111")))

(define lookup-session%
  (make-class '()
    (public
      [FRAME-WIDTH 400]
      [FRAME-HEIGHT 175]
      [NAME-WIDTH 300]
      [NUMBER-WIDTH 300]
      
      [refresh-number-info
       (let ([previous-cancel-trigger (make-trigger)])
	 (lambda ()
	   (let ([this-cancel-trigger (make-trigger)])
	     (trigger-hit previous-cancel-trigger)
	     (set! previous-cancel-trigger this-cancel-trigger)
	     (thread
	      (lambda ()
		(send number-text set-value "(Searching...)")
		(let* ([name (send name-text get-value)]
		       [home? (zero? (send number-selector get-selection))]
		       [number (lookup-number name home?)]
		       [number-string (if number
					  number
					  "(Unknown)")])
		  (unless (trigger-hit? this-cancel-trigger)
		    (send number-text set-value number-string))))))))]

      a-frame
      a-panel
      name-text
      quit-button
      number-selector
      number-text)
    
    (lambda ()
      (set! a-frame
	    (make-object wx:frame%
			 '() ; No parent
			 "Phonebook"
			 -1 -1 ; Position it anywhere
			 FRAME-WIDTH FRAME-HEIGHT))

      (send a-frame show #t)

      (set! a-panel
	    (make-object wx:panel%
			 a-frame ; Panel is in a-frame
			 0 0 FRAME-WIDTH FRAME-HEIGHT)) ; Occupy whole frame
      
      (set! name-text
	    (make-object wx:text%
			 a-panel
			 (lambda (self event) (refresh-number-info))
			 "Name"
			 ""
			 -1 -1 ; Default position
			 NAME-WIDTH -1)) ; Specific width, default height
      
      (set! quit-button
	    (make-object wx:button%
			 a-panel
			 (lambda (self event) 
			   (send a-frame show #f)) ; Simple event-handling
			 "Quit")) ; ... default position and size

      (send a-panel new-line)

      (set! number-selector
	    (make-object wx:radio-box%
			 a-panel
			 (lambda (self event) (refresh-number-info))
			 ""
			 -1 -1 -1 -1  ; default position and size
			 (list "Home Number" "Office Number")))
      
      (send a-panel new-line)

      (set! number-text
	    (make-object wx:text%
			 a-panel
			 (lambda (self event) #f) ; No event-handling
			 "Number"
			 "(Unknown)"
			 -1 -1 ; Default position
			 NUMBER-WIDTH -1)) ; Specific width, default height
      (send number-text set-editable #f))))

;; Part 5: This is the new class:

(define lookup-session-with-counter%
  (make-class lookup-session%
    (public
      [FRAME-HEIGHT 210]
      [COUNTER-WIDTH 250])
    
    (inherit a-panel)
    (rename [basic-refresh-number-info refresh-number-info])
    
    (private [search-counter 0])
    (public
      counter-text
      [refresh-number-info
       (lambda ()
	 (set! search-counter (add1 search-counter))
	 (send counter-text set-value (number->string search-counter))
	 (basic-refresh-number-info))])

    (lambda ()
      (super-init)

      (send a-panel new-line)
      (set! counter-text (make-object wx:text%
				      a-panel
				      (lambda (self event) #f)
				      "Number of Searches Started:"
				      "0"
				      -1 -1
				      COUNTER-WIDTH -1))
      (send counter-text set-editable #f))))

(make-object lookup-session%)
