
; Opening files:

(define-struct mred:handler (name extension handler))

(define mred:format-handlers '())
(define mred:mode-handlers '())

(package (mred:insert-format-handler mred:insert-mode-handler)
  (define make-handler
    (letrec ([string-list?
              (lambda (l)
                (cond
                  [(null? l) #t]
                  [(not (pair? l)) #f]
                  [else
                   (and (string? (car l))
                        (string-list? (cdr l)))]))])
      (lambda (name extension handler)
        (cond
          [(not (string? name))
           (error "mred:insert-format-handler: name was not a string")]
          [(and (not (procedure? extension))
            	   (not (string? extension))
            	   (not (string-list? extension)))
           (error (string-append "mred:insert-format-handler: "
                                 "extension was not a string,"
                                 "list of strings, or a predicate"))]
          [(not (procedure? handler))
           (error "mred:insert-format-handler: handler was not a function")]
          [else (make-mred:handler name
                                   extension
                                   handler)]))))
  
  (define mred:insert-format-handler
    (lambda args
      (set! mred:format-handlers 
            (cons (apply make-handler args)
                  mred:format-handlers))))
  
  (define mred:insert-mode-handler
    (lambda args
      (set! mred:mode-handlers 
            (cons (apply make-handler args)
                  mred:mode-handlers)))))

(constant mred:insert-format-handler)
(constant mred:insert-mode-handler)

(package (mred:std-find-format-handler mred:std-find-mode-handler)
  (define find-handler
    (lambda (name handlers)
      (catch exit
        (let* ([len (string-length name)]
               [extension
                (let loop ([p (sub1 len)])
                  (cond
                    [(negative? p) (exit #f)]
                    [(char=? (string-ref name p) #\.)
                     (substring name (add1 p) len)]
                    [else (loop (sub1 p))]))])
	  (string-lowercase! extension)
          (for-each
           (lambda (handler)
             (let ([ext (mred:handler-extension handler)])
               (if (or (and (procedure? ext)
                            (ext name ext))
                       (and (string? ext)
                            (string=? ext extension))
                       (and (pair? ext)
                            (ormap (lambda (ext) 
				     (string=? ext extension))
                                   ext)))
                   (exit (mred:handler-handler handler)))))
           handlers)
	  #f))))
  
  (define mred:std-find-format-handler
    (lambda (name)
      (find-handler name mred:format-handlers)))
  
  (define mred:std-find-mode-handler
    (lambda (name)
      (find-handler name mred:mode-handlers))))

(constant mred:std-find-format-handler)
(define mred:find-format-handler mred:std-find-format-handler)
(constant mred:std-find-mode-handler)
(define mred:find-mode-handler mred:std-find-mode-handler)

(package (mred:std-find-named-format-handler
          mred:std-find-named-mode-handler)
  (define find-named-handler
    (lambda (name handlers)
      (let loop ([l handlers])
        (cond
          [(null? l) #f]
          [(string-ci=? (mred:handler-name (car l)) name)
           (mred:handler-handler (car l))]
          [else (loop (cdr l))]))))
  
  (define mred:std-find-named-format-handler
    (lambda (name)
      (find-named-handler name mred:format-handlers)))
  
  (define mred:std-find-named-mode-handler
    (lambda (name)
      (find-named-handler name mred:mode-handlers))))

(constant mred:std-find-named-format-handler)
(define mred:find-named-format-handler mred:std-find-named-format-handler)
(constant mred:std-find-named-mode-handler)
(define mred:find-named-mode-handler mred:std-find-named-mode-handler)

(define mred:std-edit-file
  (opt-lambda (filename [frames mred:frames])
    (let ([handler
           (if (string? filename)
               (mred:find-format-handler filename)
               #f)])
      (if handler
          (handler filename)
          (let ([frame (if frames
                    		     (send frames get-active-frame)
                    		     #f)])
            (if frame
             	  (send frame open-file filename)
             	  (make-object mred:frame% filename #t frames)))))))

(constant mred:std-edit-file)
(define mred:edit-file mred:std-edit-file)

(define mred:std-open-file
  (opt-lambda ([frames mred:frames])
    (let ((file (mred:get-file)))
      (if file
       	  (mred:edit-file file frames)))))
(constant mred:std-open-file)
(define mred:open-file mred:std-open-file)

; Modes

(define mred:basic-mode%
  (make-class ()
    (public
      [name "undefined"]
      [install (lambda (edit) (void))]
      [deinstall (lambda (edit) (void))]
      
      [on-char (lambda (edit event) #f)]
      [on-event (lambda (edit event) #f)]
      
      [on-insert (lambda (edit start len) #t)]
      [after-insert (lambda (edit start len) (void))]
      [on-delete (lambda (edit start len) #t)]
      [after-delete (lambda (edit start len) (void))]
      
      [file-format wx:const-media-ff-same]
      [standard-style-delta (make-object wx:style-delta% 
                                         wx:const-change-normal)])
    (lambda () (void))))
(constant mred:basic-mode%)

(define mred:std-mode%
  (make-class mred:basic-mode%
    (public
      [name "default"]
      [keymap (make-object wx:keymap%)]
      [install 
       (lambda (edit)
	 (send keymap set-next-keymap (send edit get-keymap)))]
      [on-char
       (lambda (edit event)
	 (send keymap handle-key-event edit event))]
      [on-event
       (lambda (edit event)
	 (send keymap handle-mouse-event edit event))])
    (lambda ()
      (super-init)
      (mred:set-keymap-error-handler keymap)
      (mred:set-keymap-implied-shifts keymap))))
(constant mred:std-mode%)
(define mred:mode% mred:std-mode%)

(mred:insert-mode-handler "None" () (lambda args #f))

; load-with-cd

(define load-with-cd
  (lambda (file)
    (let ([old-wd (wx:get-working-directory)]
       	  [new-wd (wx:path-only file)]
       	  [filename (wx:file-name-from-path file)])
      (dynamic-wind
       (lambda ()
       	 (if (not (null? new-wd))
       	     (wx:set-working-directory new-wd)))
       (lambda ()
       	 (load filename))
       (lambda ()
       	 (if (not (null? new-wd))
       	     (wx:set-working-directory old-wd)))))))
(constant load-with-cd)

(define load/cd load-with-cd)
(constant load/cd)

(define normalize-path
  (let ([dir-separator (if (eq? mred:platform 'windows) "\\" "/")])
    (opt-lambda (file [wrt #f])
      (let ([name
	     (cond
	      [(and (eq? mred:platform 'windows)
		    (>= (string-length file) 2)
		    (string=? ":" (substring file 1 2)))
	       (wx:expand-path file)]
	      [(or (string=? dir-separator (substring file 0 1))
		   (and (eq? mred:platform 'unix)
			(string=? "~" (substring file 0 1))))
	       (wx:expand-path
		(if (eq? mred:platform 'windows)
		    (let ([wd (if wrt
				  wrt
				  (wx:get-working-directory))])
		      (if (and (string? wd)
			       (>= (string-length wd) 2)
			       (string=? ":" (substring wd 1 2)))
			  (string-append (substring wd 0 2)
					 file)
			  file))
		    file))]
	      [else
	       (let ([wd (if wrt
			     wrt
			     (wx:get-working-directory))])
		 (if (string? wd)
		     (normalize-path (string-append wd
						    dir-separator
						    file))
		     file))])])
	(if (eq? mred:platform 'windows)
	    (string-lowercase! (if (eq? name file)
				   (string-copy name)
				   name)))
	name))))
(constant normalize-path)
