
(package (mred:common-put-file 
	  mred:common-get-file 
	  mred:current-find-file-directory)

(define last-directory #f)

(define mred:current-find-file-directory
  (opt-lambda ([dir 'get])
    (cond
     [(eq? dir 'get)
      (if (not last-directory)
	  (set! last-directory (wx:get-working-directory)))
      last-directory]
     [(and (string? dir)
	   (wx:dir-exists? dir))
      (set! last-directory dir)
      #t]
     [else #f])))

(define dir-separator (if (eq? mred:platform 'windows) "\\" "/"))
(define wildcard (if (eq? mred:platform 'windows) "*.*" "*"))

(define strip-separator
  (let ([sep-char (string-ref dir-separator 0)])
    (lambda (name)
      (if (equal? name "")
	  name
	  (let ([last (string-ref name (sub1 (string-length name)))])
	    (if (char=? last sep-char)
		(substring name 0 (sub1 (string-length name)))
		name))))))

(define mred:finder-dialog%
  (make-class wx:dialog-box%
	      (inherit
	       new-line tab fit
	       show
	       popup-menu)

	      (private
	       save-mode?
	       replace-ok?

	       dir-choice
	       goto-button

	       name-list
	       name-field
	       into-dir-button
	       ok-button
	       cancel-button

	       dirs current-dir
	       last-selected

	       result-box)

	      (private
	       [set-directory
		(lambda (dir)
		  (let ([dir (strip-separator dir)])
		    (set! current-dir dir)
		    (set! last-directory dir)
		    (let ([lists
			   (let loop ([this-dir dir]
				      [dir-list ()]
				      [menu-list ()])
			     (let ([dir-name 
				    (wx:file-name-from-path this-dir)])
			       (if (and (string? dir-name)
					(not (string=? "" dir-name)))
				   (let ([in-dir (wx:path-only this-dir)])
				     (if (eq? mred:platform 'windows)
					 (string-lowercase! in-dir))
				     (let* ([dir-list (cons this-dir dir-list)]
					    [menu-list 
					     (cons dir-name menu-list)])
				       (if (and (string? in-dir)
						(not (string=? "" in-dir)))
					   (loop in-dir dir-list menu-list)
					   (cons dir-list menu-list))))
				   (cons dir-list menu-list))))])
		      
		      (set! dirs (cons (normalize-path dir-separator 
						       current-dir)
				       (car lists)))
		      
		      (send dir-choice clear)
		      (let loop ([choices (cons (normalize-path dir-separator
								current-dir)
						(cdr lists))])
			(unless (null? choices)
			    (let ([choice (car choices)])
			      (if (eq? mred:platform 'windows)
				  (string-lowercase! choice))
			      (send dir-choice append choice)
			      (loop (cdr choices)))))
		      (send dir-choice set-selection (sub1 (length dirs)))
		      (send dir-choice set-size -1 -1 -1 -1))
		    
		    (send name-list clear)
		    (send name-list set
			  (quicksort
			   (let ([list
				  (let loop ([name (wx:find-first-file
						    (string-append 
						     dir 
						     dir-separator
						     wildcard)
						    (if (eq? mred:platform 
							     'windows)
							2
							0))])
				    (if (string? name)
					(let ([file
					       (wx:file-name-from-path name)])
					  (if (and (not (equal? file ".."))
						   (not (equal? file ".")))
					      (begin
						(if (eq? mred:platform
							 'windows)
						    (string-lowercase! file))
						(cons 
						 file
						 (loop (wx:find-next-file))))
					      (loop (wx:find-next-file))))
					'()))])
			     (map (lambda (x)
				    (if (wx:dir-exists?
					 (normalize-path x current-dir))
					(string-append x dir-separator)
					x))
				  list))
			   string<?))
		    (set! last-selected -1)))])

	      (public
	       [do-dir
		(lambda (choice event)
		  (let ([which (send event get-selection)])
		    (if (< which (length dirs))
			(set-directory (list-ref dirs which)))))]

	       [do-goto
		(lambda (button event)
		  (let ([dir (wx:get-text-from-user
			      "Directory" "Go to Directory")])
		    (if (string? dir)
			(let ([dir (normalize-path dir current-dir)])
			  (if (wx:dir-exists? dir)
			      (set-directory dir)
			      (wx:message-box 
			       (string-append "Bad directory: " dir)
			       "Error"))))))]

	       [on-default-action
		(lambda (which)
		  (if (eq? which name-list)
		      (let* ([which (send name-list get-string-selection)]
			     [dir (string-append current-dir
						 dir-separator
						 which)])
			(if (wx:dir-exists? dir)
			    (set-directory dir)
			    (if save-mode?
				(send name-field set-value which)
				(do-ok))))
		      (if (eq? which name-field)
			  (do-ok))))]

	       [do-name
		(lambda (text event)
		  (if (eq? (send event get-event-type)
			   wx:const-event-type-text-enter-command)
		      (do-ok)))]
	       [do-name-list
		(lambda args #f)]

	       [do-into-dir
		(lambda args
		  (let ([name (send name-list get-string-selection)])
		    (if (string? name)
			(let ([name (string-append current-dir
						   dir-separator
						   name)])
			  (if (wx:dir-exists? name)
			      (set-directory name))))))]

	       [do-ok
		(lambda args
		  (let ([name
			 (if save-mode?
			     (send name-field get-value)
			     (send name-list get-string-selection))])
		    (cond
		     [(not (string? name)) 'nothing-selected]
		     [(string=? name "")
		      (wx:message-box "You must specify a file name"
				      "Error")]
		     [else
		      (let ([file (string-append current-dir
						 dir-separator
						 name)])
			(if (wx:dir-exists? file)
			    (if save-mode?
				(wx:message-box
				 "That is the name of a directory."
				 "Error")
				(set-directory file))
			    (if (or (not save-mode?)
				    (not (wx:file-exists? file))
				    replace-ok?
				    (= (wx:message-box
					(string-append
					 "The file " name " already exists. "
					 "Replace it?")
					"Warning"
					wx:const-yes-no)
				       wx:const-yes))
				(begin
				  (set-box! result-box file)
				  (show #f)))))])))]

	       [do-cancel
		(lambda args
		  (set-box! result-box #f)
		  (show #f))])
	      (lambda (save? replace? result-value-box start-dir 
			     start-name prompt)

		(super-init () "Find File" #t 300 300 500 500)

		(set! save-mode? save?)
		(set! result-box result-value-box)
		(set! replace-ok? replace?)

		(make-object wx:message% this prompt)

		(new-line)

		(set! dir-choice
		      (make-object wx:choice%
				   this do-dir ()))

		(new-line)

		(set! name-list
		      (make-object wx:list-box%
				   this do-name-list
				   () wx:const-single
				   -1 -1
				   500 300))
		(new-line)

		(when save?
		    (set! name-field
			  (make-object wx:text%
				       this do-name
				       "Name: " ""
				       -1 -1
				       400 -1
				       wx:const-process-enter))
		    (if (string? start-name)
			(send name-field set-value start-name))
		    (new-line))

		(when save?
		      (set! into-dir-button
			    (make-object wx:button%
					 this do-into-dir "Open Directory")))

		(set! goto-button
		      (make-object wx:button%
				   this do-goto "Go to Directory..."))

		(tab 100)

		(set! cancel-button
		      (make-object wx:button%
				   this do-cancel
				   "Cancel"))

		(let ([cancel-width (box 0)]
		      [cancel-height (box 0)])
		  (send cancel-button get-size cancel-width cancel-height)
		  (set! ok-button
			(make-object wx:button%
				     this do-ok
				     "OK" -1 -1 (unbox cancel-width))))
		(fit)

		(cond
		 [(and start-dir
		       (not (null? start-dir))
		       (wx:dir-exists? start-dir))
		  (set-directory start-dir)]
		 [last-directory (set-directory last-directory)]
		 [else (set-directory (wx:get-working-directory))])

		(show #t))))

(define mred:common-put-file
  (opt-lambda ([name ()][directory ()][replace? #f]
				[prompt "Select File"])
    (let* ([directory (if (and (null? directory)
			       (string? name))
			  (wx:path-only name)
			  directory)]
	   [name (if (string? name)
		     (wx:file-name-from-path name)
		     name)]
	   [v (box "")])
      (make-object mred:finder-dialog% #t replace? v 
		   directory name prompt)
      (unbox v))))

(define mred:common-get-file
  (opt-lambda ([directory ()][prompt "Select File"])
    (let ([v (box "")])
      (make-object mred:finder-dialog% #f #f v directory '() prompt)
      (unbox v))))

)
(constant mred:common-put-file)
(constant mred:common-get-file)

(define mred:std-put-file
  (opt-lambda ([name ()][directory ()][replace? #f][prompt "Select File"])
     (let* ([directory (if (and (null? directory)
				(string? name))
			   (wx:path-only name)
			   directory)]
	    [name (if (string? name)
		      (wx:file-name-from-path name)
		      name)]
	    [f (wx:file-selector prompt directory name)])
       (if (null? f)
	   #f
	   (let* ([f (normalize-path f)]
		  [dir (wx:path-only f)]
		  [name (wx:file-name-from-path f)])
	     (cond
	      [(not (wx:dir-exists? dir))
	       (wx:message-box "Error" "That directory does not exist.")
	       #f]
	      [(equal? name "")
	       (wx:message-box "Error" "Empty filename.")
	       #f]
	      [else f]))))))
(constant mred:std-put-file)

(define mred:std-get-file
  (opt-lambda ([directory ()][prompt "Select File"])
	      (let ([f (wx:file-selector prompt directory)])
		(if (null? f)
		    #f
		    (let ([f (normalize-path f)])
		      (cond
		       [(wx:dir-exists? f)
			(wx:message-box "Error" "That is a directory name.")
			#f]
		       [(not (wx:file-exists? f))
			(wx:message-box "That file does not exist.")
			#f]
		       [else f]))))))
(constant mred:std-get-file)

(define mred:put-file mred:std-put-file)
(define mred:get-file mred:std-get-file)
