
;;; Define the standard editing window class

;; TODO:
;;  * open-local

(define mred:frame-name "MrEd")

; A refinement of wx:media-edit% that performs parenthesis-matching
;  and handles modes
(define mred:std-edit%
  (make-class wx:media-edit%
    (inherit flash-on get-keymap get-start-position
	     on-default-char on-default-event 
	     set-file-format get-style-list)
    (public
      [auto-set-wrap? #f]
      [set-auto-set-wrap
       (lambda (v)
	 (set! auto-set-wrap? v)
	 (if (not v)
	     ((ivar set-max-width) -1)))]
      [mode #f]
      [set-mode
       (lambda (m)
	 (if mode
	     (send mode deinstall this))
	 (if (is-a? m mred:mode%)
	     (begin
	       (set! mode m)
	       (set-file-format (ivar m file-format))
	       (send (send (get-style-list) 
			   find-named-style "Standard")
		     set-delta (ivar m standard-style-delta))
	       (send m install this))
	     (begin
	       (set! mode #f)
	       (send (send (get-style-list) 
			   find-named-style "Standard")
		     set-delta (make-object wx:style-delta%)))))]
      [on-local-event
       (lambda (mouse)
	 (if mode
	     (unless (send mode on-event this mouse)
	       (on-default-event mouse))
	     ((ivar super on-local-event) mouse)))]
      [on-local-char
       ; mode & paren balancing
       (lambda (key)
	 (unless (or (and mode (send mode on-char this key))
		     (and (not mode)
			  (send (get-keymap) handle-key-event 
				this key)))
	   (on-default-char key)
	   (let ((code (send key get-key-code)))
	     ; close paren is ASCII 41
	     ; close square bracket is 93
	     ; quote is 34
	     (if (or (= 41 code)
		     (= 34 code)
		     (= 93 code))
		 (let ([pos (mred:scheme-backward-match 
			     this
			     (get-start-position)
			     0)])
		   (if pos
		       (flash-on pos (+ 1 pos))))))))]
      [on-insert
       (lambda (start len)
	 (if (or (not mode) (send mode on-insert this start len))
	     ((ivar super on-insert) start len)))]
      [on-delete
       (lambda (start len)
	 (if (or (not mode) (send mode on-delete this start len))
	     ((ivar super on-delete) start len)))]
      
      [after-insert
       (lambda (start len)
	 (if mode (send mode after-insert this start len))
	 ((ivar super after-insert) start len))]
      
      [after-delete
       (lambda (start len)
	 (if mode (send mode after-delete this start len))
	 ((ivar super after-delete) start len))])
    
    (lambda args
      (apply-super-init args)
      (mred:set-keymap (get-keymap)))))
(constant mred:std-edit%)
(define mred:edit% mred:std-edit%)


(define mred:std-pasteboard%
  (make-class wx:media-pasteboard%
     (public
      [mode #f]
      [set-mode (lambda args #f)]
      [auto-set-wrap? #f]
      [set-auto-set-wrap
       (lambda (v)
	 (set! auto-set-wrap? v)
	 (if (not v)
	     ((ivar set-max-width) -1)))])))
(constant mred:std-pasteboard%)
(define mred:pasteboard% mred:std-pasteboard%)

(define mred:std-canvas%
  (make-class wx:media-canvas%
    (class
      [edit% mred:edit%])
    (opt-lambda (parent [x -1] [y -1] [w -1] [h -1]
			[name ""] [style 0] [spp 100] [m ()])
      (super-init parent x y w h
		  name style spp
		  (if (null? m)
		      (make-object edit%)
		      m)))))
(constant mred:std-canvas%)
(define mred:canvas% mred:std-canvas%)

; This defines a simple editing frame without a menu bar.
; On initialization, it doesn't automatically show itself
(define mred:simple-frame%
  (make-class wx:frame%
    (inherit get-client-size get-title set-title)
    ; Overrideable constants
    (class
      [WIDTH 600]
      [HEIGHT 600]
      [MARGIN 2]
      
      [TOP-MARGIN 0]
      [BOTTOM-MARGIN 0]
      [LEFT-MARGIN 0]
      [RIGHT-MARGIN 0])
    ; Classes for subparts
    ; A window contains a canvas for drawing the text,
    ;  plus an object for the the text itself. There's
    ;  also a menu bar with some menus.
    (class
      [canvas% mred:canvas%])
    ; instance variables
    (public
      edit
      canvas last-focus-canvas
      canvases
      [title-prefix ""])

    ; we need to cal this one directly
    (public
     [on-simple-size 
       ; Window resize -> resize canvases
       (lambda ignored
	 (if (not (void? canvas))
	     (let ((client-w (box 0))
		   (client-h (box 0)))
	       (get-client-size client-w client-h)
	       (let* ([w (- (unbox client-w) (* 2 MARGIN)
			    LEFT-MARGIN RIGHT-MARGIN)]
		      [h (- (unbox client-h) (* 2 MARGIN)
			    TOP-MARGIN BOTTOM-MARGIN)]
		      [count (length canvases)]
		      [h (/ h count)])
		 (let loop ([canvases canvases]
			    [v (+ MARGIN TOP-MARGIN)])
		   (unless (null? canvases)
		     (send (car canvases) set-size 
			   (+ MARGIN LEFT-MARGIN) v
			   w h)
		     (let ([edit (send (car canvases) get-media)])
		       (if (ivar edit auto-set-wrap?)
			   (let ([admin (send edit get-admin)]
				 [w-box (box 0)]
				 [h-box (box 0)])
			     (unless (null? admin)
			        (send admin get-view () () w-box h-box #f)
				(send edit set-max-width (unbox w-box))))))
		     (loop (cdr canvases) (+ v h))))))))])

    ; methods
    (public
      [on-frame-active (lambda () (void))]
      [set-title-prefix
       (lambda (s)
	 (if (string? s)
	     (set! title-prefix s)))]
      [make-canvas
       (lambda ()
	 (let ([canvas
		(make-object
		 (make-class canvas%
		   (public
		     [set-frame-title
		      (lambda()
			(let* ([edit (send this get-media)]
			       [title
				(if (null? edit)
				    "MrEd"
				    (send edit 
					  get-filename))]
			       [title
				(if (string? title)
				    (string-append
				     title-prefix
				     (wx:file-name-from-path
				      title))
				    title)])
			  (if (string? title)
			      (let ([old-title (get-title)])
				(if (not (equal? title 
						 old-title))
				    (set-title title))))))]
		     [on-set-focus
		      (lambda ()
			(set! last-focus-canvas this)
			(on-frame-active)
			(set-frame-title)
			((ivar super on-set-focus)))]
		     [set-media
		      (lambda (m)
			((ivar super set-media) m)
			(set-frame-title))]))
		 this)])
	   (if (null? (send canvas get-media))
	       (send canvas set-media (make-object mred:edit%)))
	   canvas))]
      
      [on-size
       (lambda args
	 (apply on-simple-size args))]
      
      [on-close
       (lambda ()
	 #t)]
      
      [add-canvas
       (lambda ()
	 (let ([canvas (make-canvas)])
	   (set! canvases (append canvases (list canvas)))
	   (on-size)
	   canvas))]
      [remove-canvas
       (lambda (canvas)
	 (set! canvases
	       (let loop ([l canvases])
		 (cond
		  [(null? l) '()]
		  [(eq? canvas (car l))
		   (send canvas set-size 50000 50000 10 10)
		   (loop (cdr l))]
		  [else (cons (car l) (loop (cdr l)))])))
	 (on-size))]

      [active-canvas
       (lambda ()
	 (let loop ([canvases canvases])
	   (cond
	     [(null? canvases) last-focus-canvas]
	     [(send (car canvases) is-focus-on?) (car canvases)]
	     [else (loop (cdr canvases))])))]
      [active-edit
       (lambda ()
	 (send (active-canvas) get-media))])
    
    ; Initialization:
    (opt-lambda ([name mred:frame-name])
      (super-init () name -1 -1 WIDTH HEIGHT
		  (+ wx:const-default-frame wx:const-sdi)
		  name)
      
      (set! canvas (make-canvas))
      (set! last-focus-canvas canvas)
      
      (set! edit (send canvas get-media))
      
      (set! canvases (list canvas))

      ((ivar set-icon) mred:icon)

      (on-simple-size))))
(constant mred:simple-frame%)

; This defines the standard, plain-file editing window.
(define mred:std-frame%
  (let-enumerate
      ([FILE-NEW-FRAME
	FILE-LOCAL-OPEN
	FILE-NEW
	FILE-SAVE
	FILE-SAVE-AS-TEXT
	FILE-SAVE-AS-FORMATTED
	FILE-SAVE-AS
	FILE-RELOAD
	FILE-CLOSE
	FILE-SPLIT
	FILE-COLLAPSE
	FILE-SWITCH-TO
	FILE-PRINT
	FILE-MODE

	EDIT-WRAPPING

	END-OF-MENUS])
    (make-class mred:simple-frame%
      (inherit edit active-edit active-canvas 
	       show add-canvas remove-canvas canvases)
      (class
	[edit% mred:edit%]
	[menu% mred:menu%]
	[menu-bar% mred:menu-bar%]
	[allow-split? #t])
      (private 
	edit-offset 
	font-offset
	other-offset
	keep-buffers?
	
	exit-callback-tag
	
	frames
	buffers
	
	[save-as
	 (lambda (format)
	   (let ([file (mred:put-file)])
	     (if file
		 (send edit save-file file format))))])
      
      (public
        [on-frame-active
	 (lambda ()
	   (if keep-buffers?
	       (send frames set-active-frame this)))]
	[check-saved
	 (lambda (canvas)
	   (let* ([edit (send canvas get-media)]
		  [name (send edit get-filename)]
		  [name (if (string? name)
			    name
			    "Untitled")])
	     (or (not (send edit modified?))
		 (eq? 'continue (mred:unsaved-warning name "Close")))))]
	[get-frame-group
	 (lambda ()
	   (if keep-buffers?
	       frames
	       #f))]
	[make-edit
	 (lambda ()
	   (make-object edit%))]
	[next-menu-id
	 (lambda ()
	   other-offset)]
	[open-file
	 (lambda (filename)
	   ; filename = () => ask user
	   ; filename = #f => no file, make untitled
	   ; otherwise, filename = name string
	   (if (or keep-buffers?
		   (check-saved (active-canvas)))
	       (let* ([filename
		       (if (null? filename)
			   (mred:get-file)
			   filename)]
		      ; at this point, name is either a string or #f
		      ; Is the buffer going to be untitled?
		      [untitled (not filename)]
		      ; If filename is still #f, make up a name
		      [filename (if filename
				    filename
				    (if keep-buffers?
					(send buffers 
					      make-untitled-name)
					"Untitled"))]
		      ; Look for an existing buffer for this file
		      [edit
		       (if (or untitled (not keep-buffers?))
			   #f
			   (send buffers find-buffer 'file
				 (lambda (name edit)
				   (let ([name (send edit 
						     get-filename)])
				     (and (not (null? name))
					  (string=? filename 
						    name))))))])
		 ; If we didn't find a buffer, create one
		 (if (not edit)
		     (let ([edit (make-edit)])
		       ; Load in the file, if it exists
		       (if (and (not untitled)
				(wx:file-exists? filename))
			   (send edit load-file filename)
			   (begin
			     (send edit erase)
			     (send edit set-filename filename 
				   untitled)))
		       ; Pick the editing mode
		       (let* ([filename (send edit get-filename)]
			      [mode-handler (if (null? filename)
						#f
						(mred:find-mode-handler
						 filename))])
			 (if mode-handler
			     (let ([mode (mode-handler edit)])
			       (send edit set-mode mode))
			     (send edit set-mode #f)))
		       ; We created this buffer; add to the global list
		       (if keep-buffers?
			   (send buffers add-buffer 'file #f edit))
		       (send (active-canvas) set-media edit))
		     ; Found an existing edit
		       (begin
			 (send buffers reset-buffer-state edit)
			 (send (active-canvas) set-media edit))))))]
	[on-menu-command
	 ; Handle menu selections
	 (lambda (op)
	   (let ([edit (active-edit)])
	     (cond
	       [(>= op font-offset)
		(send edit do-font (- op font-offset))]
	       [(>= op edit-offset)
		(send edit do-edit (- op edit-offset))]
	       [(= op FILE-NEW-FRAME) (mred:edit-file #f)]
	       [(= op FILE-SWITCH-TO)
		(send buffers pick (active-canvas))]
	       [(= op FILE-LOCAL-OPEN)
		(let ((file (mred:get-file)))
		  (if file
		      (open-file file)))]
	       [(= op FILE-NEW)
		(let ((file (mred:put-file)))
		  (if file
		      (open-file file)))]
	       [(= op FILE-SAVE)
		(send edit save-file)]
	       [(= op FILE-SAVE-AS-TEXT)
		(save-as wx:const-media-ff-text)]
	       [(= op FILE-SAVE-AS-FORMATTED)
		(save-as wx:const-media-ff-std)]
	       [(= op FILE-SAVE-AS)
		(save-as wx:const-media-ff-same)]
	       [(= op FILE-RELOAD) (send edit load-file)]
	       [(= op FILE-CLOSE)
		(if (on-close)
		    (show #f))]
	       [(= op FILE-PRINT)
		(send (active-edit) print '())]
	       [(= op FILE-SPLIT)
		(let ([new-canvas (add-canvas)])
		  (send new-canvas set-media edit))]
	       [(= op FILE-COLLAPSE)
		(when (> (length canvases) 1)
		   (let ([canvas (active-canvas)])
		     (if (or keep-buffers?
			     (check-nonunique-or-saved canvas))
			 (remove-canvas canvas))))]
	       [(= op FILE-MODE)
		(let* ([modes (map mred:handler-name mred:mode-handlers)]
		       [name (wx:get-single-choice
			      "Select a Mode" "Mode"
			      modes)])
		  (unless (null? name)
		     (let* ([handler (mred:find-named-mode-handler name)]
			    [mode (handler edit)])
		       (send edit set-mode mode))))]
	       [(= op EDIT-WRAPPING)
		(send edit set-auto-set-wrap
		      (not (ivar edit auto-set-wrap?)))
		((ivar on-simple-size))])))]
	
	[check-nonunique-or-saved
	 (lambda (canvas)
	   (let ([e (send canvas get-media)])
	     (if (ormap
		  (lambda (c)
		    (and (not (eq? c canvas))
			 (eq? e (send c get-media))))
		  canvases)
		 #t
		 (check-saved canvas))))]

	[check-all-saved
	 (lambda ()
	   (andmap check-saved canvases))]
	
	[on-close
	 (lambda ()
	   (if (and ((ivar super on-close))
		    (or (not keep-buffers?)
			(send frames remove-frame this)))
	       (if keep-buffers?
		   (begin
		     (for-each (lambda (canvas)
				 (send canvas set-media '()))
			       canvases)
		     #t)
		   (if (check-all-saved)
		       (begin
			 (mred:remove-exit-callback
			  exit-callback-tag)
			 #t)
		       #f))
	       #f))]

	[make-menu-bar
	 (lambda ()
	   (let ([file-menu (make-object menu%)]
		 [edit-menu (make-object menu%)])
	     (send file-menu append FILE-LOCAL-OPEN "Open...")
	     (send file-menu append FILE-NEW "New File...")
	     (if keep-buffers?
		 (send file-menu append FILE-SWITCH-TO "Switch to..."))
	     (send file-menu append FILE-RELOAD "Revert")
	     (send file-menu append-separator)
	     (send file-menu append FILE-NEW-FRAME "New Frame")
	     (send file-menu append-separator)
	     (send file-menu append FILE-SAVE "Save")
	     (send file-menu append FILE-SAVE-AS "Save As...")
	     (send file-menu append FILE-SAVE-AS-TEXT "Save As Text...")
	     (send file-menu append FILE-SAVE-AS-FORMATTED
		   "Save As Formatted...")
	     (send file-menu append FILE-PRINT "Print...")
	     (when allow-split?
		   (send file-menu append-separator)
		   (send file-menu append FILE-SPLIT "Split")
		   (send file-menu append FILE-COLLAPSE "Collapse"))
	     (send file-menu append FILE-MODE "Set Mode...")
	     (send file-menu append-separator)
	     (send file-menu append FILE-CLOSE "Close")
	     
	     ; Use the built-in menu item lists for the edit menu:
	     (set! edit-offset END-OF-MENUS)
	     (set! font-offset
		   (send (active-edit) append-edit-items 
			 edit-menu edit-offset))
	     (send edit-menu append-separator)
	     (send edit-menu append EDIT-WRAPPING "Toggle Wrap Text")
	     (send edit-menu append-separator)
	     (set! other-offset
		   (send (active-edit) append-font-items 
			 edit-menu font-offset))

	     (make-object menu-bar%
			  (list file-menu edit-menu)
			  (list "File" "Edit"))))])
      
      (opt-lambda ([filename #f][show? #t][frameset mred:frames])
	(super-init)
	
	(set! keep-buffers? (is-a? frameset mred:frame-group%))
	
	; Build and install the menu bar:
	(let ([menu-bar (make-menu-bar)])
	  ((ivar set-menu-bar) menu-bar))
	
	(if show?
	    (show #t))
	
	(if keep-buffers?
	    (begin
	      (set! frames frameset)
	      (set! buffers (ivar frames buffers))
	      (send frames insert-frame this))
	    (begin
	      (set! exit-callback-tag
		    (mred:insert-exit-callback check-all-saved))))

	(let ([filename (if (string? filename)
			    (normalize-path filename)
			    filename)])
	  (open-file filename))
	
	(set! edit (active-edit))))))
(constant mred:std-frame%)
(define mred:frame% mred:std-frame%)


(define mred:std-pasteboard-frame%
  (make-class mred:std-frame%
      (public
       [edit% mred:pasteboard%])))
(constant mred:std-pasteboard-frame%)
(define mred:pasteboard-frame% mred:std-pasteboard-frame%)
