
/* WARNING: These classes are not garbage-collected. */

class wxChangeRecord /* : public wxObject - uncomment to GC */
{
 public:
  wxChangeRecord(void);
  virtual ~wxChangeRecord();
  virtual Bool Undo(wxMediaBuffer *media);
};

class wxUnmodifyRecord : public wxChangeRecord
{
 public:
  Bool Undo(wxMediaBuffer *media);
};

class wxInsertRecord : public wxChangeRecord
{
 private:
  long start, end;
  Bool continued;

 public:
  wxInsertRecord(long position, long length, Bool cont);

  Bool Undo(wxMediaBuffer *media);
};

class wxInsertSnipRecord : public wxChangeRecord
{
 private:
  wxSnip *snip;
  Bool continued;

 public:
  wxInsertSnipRecord(wxSnip *, Bool cont);

  Bool Undo(wxMediaBuffer *media);
};

class wxDeleteRecord : public wxChangeRecord
{
 private:
  Bool continued;
  long start, end;
  wxList *deletions;
  wxList *clickbacks;
  Bool undid;

 public:
  wxDeleteRecord(long start, long end, Bool cont);
  ~wxDeleteRecord();

  void InsertSnip(wxSnip *);
  void AddClickback(wxClickback *);

  Bool Undo(wxMediaBuffer *media);
};

class wxDeleteSnipRecord : public wxChangeRecord
{
 private:
  Bool continued;
  wxList *deletions;

 public:
  Bool undid;

  wxDeleteSnipRecord(Bool cont);
  ~wxDeleteSnipRecord();

  void InsertSnip(wxSnip *, wxSnip *, float, float);

  Bool Undo(wxMediaBuffer *media);
};

class wxStyleChangeRecord : public wxChangeRecord
{
 private:
  Bool continued;
  long start, end;
  wxList *changes;

 public:
  wxStyleChangeRecord(long start, long end, Bool cont);
  ~wxStyleChangeRecord();

  void AddStyleChange(long start, long end, wxStyle *style);
  Bool Undo(wxMediaBuffer *media);
};

class wxStyleChangeSnipRecord : public wxChangeRecord
{
 private:
  Bool continued;
  wxList *changes;

 public:
  wxStyleChangeSnipRecord(Bool cont);
  ~wxStyleChangeSnipRecord();

  void AddStyleChange(wxSnip *, wxStyle *style);
  Bool Undo(wxMediaBuffer *media);
};

class wxMoveSnipRecord : public wxChangeRecord
{
 private:
  Bool continued;
  wxSnip *snip;
  float x, y;
  Bool delta;
 public:
  wxMoveSnipRecord(wxSnip *snip, float x, float y, Bool delta, Bool cont);
  Bool Undo(wxMediaBuffer *media);
};

class wxResizeSnipRecord : public wxChangeRecord
{
 private:
  Bool continued;
  wxSnip *snip;
  float x, y;
 public:
  wxResizeSnipRecord(wxSnip *snip, float x, float y, Bool cont);
  Bool Undo(wxMediaBuffer *media);
};

