
#ifndef wx_keym
#define wx_keym

#include "wx_stdev.h"
#include "wx_hash.h"

class wxMediaBuffer;

typedef Bool (*wxKeyFunction)(wxMediaBuffer *media, wxKeyEvent &event, 
			      void *data);
typedef Bool (*wxMouseFunction)(wxMediaBuffer *media, wxMouseEvent &event, 
				void *data);

enum {
  wxKEY_FINAL = 0,
  wxKEY_PREFIX
};

typedef void (*wxKeyErrorFunction)(void *data, char *str);

#define WXK_MOUSE_RIGHT         (-1)
#define WXK_MOUSE_LEFT          (-2)
#define WXK_MOUSE_MIDDLE        (-3)
#define WXK_MOUSE_RIGHT_DOUBLE  (-4)
#define WXK_MOUSE_LEFT_DOUBLE   (-5)
#define WXK_MOUSE_MIDDLE_DOUBLE (-6)
#define WXK_MOUSE_RIGHT_TRIPLE  (-7)
#define WXK_MOUSE_LEFT_TRIPLE   (-8)
#define WXK_MOUSE_MIDDLE_TRIPLE (-9)

#define WXK_CLICK_ADDER         (-3)

class wxKeymap : public wxObject
{
  wxHashTable *keyfunctions, *mousefunctions;
  wxHashTable *keys;
  wxKeyErrorFunction err;
  void *errdata;

  long numImpliedShifts, allocedImplies;
  long *impliesShift;
  int usage;

  wxKeymap *chainTo;

  long doubleInterval;

  int lastButton;
  int clickCount;
  long lastTime;
  float lastX, lastY;

  class wxKeycode *prefix;

  char *active_mouse_function;

  class wxKeycode *FindKey(long, Bool, Bool, Bool, Bool, class wxKeycode *);
  Bool HandleEvent(long code, Bool shift, Bool ctrl, Bool alt, Bool meta,
		   char **fname);

 public:
  wxKeymap();
  ~wxKeymap();
  
  Bool HandleKeyEvent(wxMediaBuffer *media, wxKeyEvent &event);
  Bool HandleMouseEvent(wxMediaBuffer *media, wxMouseEvent &event);

  void BreakSequence(void);

  class wxKeycode *MapFunction(long code, Bool shift, Bool ctrl, 
			       Bool alt, Bool meta, 
			       char *fname, class wxKeycode *prevkey=NULL, 
			       int keytype = wxKEY_FINAL);
  void MapFunction(char *keyname, char *fname);

  void ImpliesShift(char *str);

  void AddKeyFunction(char *name, wxKeyFunction func, void *data);
  void AddMouseFunction(char *name, wxMouseFunction func, void *data);
  Bool CallFunction(char *name, wxMediaBuffer *media, wxKeyEvent &event);
  Bool CallFunction(char *name, wxMediaBuffer *media, wxMouseEvent &event);

  void SetErrorCallback(wxKeyErrorFunction, void *d=NULL);

  void SetNextKeymap(wxKeymap *);

  void AdjustUsage(Bool newUser);
  Bool IsUsed(void);

  long GetDoubleClickInterval();
  void SetDoubleClickInterval(long);
};

#endif
